""" Subtyping support
"""
from zope.interface import implements
from zope.interface import alsoProvides, noLongerProvides
from zope.event import notify
from zope.publisher.interfaces import NotFound

from Products.statusmessages.interfaces import IStatusMessage
from Products.Five.browser import BrowserView
#from Products.CMFCore.utils import getToolByName
from eea.uberlisting.interfaces import IUberlistingEnabled 
from eea.uberlisting.subtypes.interfaces import IUberlistingSubtyper

from eea.uberlisting.events import (
    UberlistingWillBeDisabledEvent,
    UberlistingWillBeEnabledEvent,
    UberlistingDisabledEvent,
    UberlistingEnabledEvent,
)

from eea.uberlisting import EEAMessageFactory as _

class UberlistingPublicSubtyper(BrowserView):
    """ Public support for subtyping objects
        view for non IPossibleUberlistingNavigable objects
    """
    implements(IUberlistingSubtyper)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def _redirect(self, msg=''):
        """ Redirect
        """
        if self.request:
            if msg:
                IStatusMessage(self.request).addStatusMessage(msg, type='info')
            self.request.response.redirect(self.context.absolute_url())
        return msg

    @property
    def can_enable(self):
        """ See IUberlistingSubtyper
        """
        return False

    @property
    def can_disable(self):
        """ See IUberlistingSubtyper
        """
        return False

    @property
    def is_uberlisted(self):
        """ Is faceted navigable?
        """
        return False

    def enable(self):
        """ See IUberlistingSubtyper
        """
        raise NotFound(self.context, 'enable', self.request)

    def disable(self):
        """ See IUberlistingSubtyper
        """
        raise NotFound(self.context, 'disable', self.request)


class UberlistingSubtyper(UberlistingPublicSubtyper):
    """ Support for subtyping objects
        view for IPossibleUberlistingNavigable objects
    """

    #def __bobo_traverse__(self, REQUEST, name):
    #    """ Bobo traverse 
    #    """
    #    import pdb; pdb.set_trace()
    #    res = getattr(self, name)
    #    if res:
    #        return res
    #    return UberlistingSubtyper.__bobo_traverse__(self, REQUEST, name)

    #def publishTraverse(self, request, name):
    #    """ Custom traversal to allow cropImage to be called
    #    """
    #    import pdb; pdb.set_trace()
    #    #if name == "cropImage":
    #    #    return self.cropImage
    #    return super(UberlistingSubtyper, self).publishTraverse(request, name)

    @property
    def can_enable(self):
        """ See IUberlistingSubtyper
        """
        return not self.is_uberlisted

    @property
    def can_disable(self):
        """ See IUberlistingSubtyper
        """
        return self.is_uberlisted

    @property
    def is_uberlisted(self):
        """ Is faceted navigable?
        """
        return IUberlistingEnabled.providedBy(self.context)

    def enable(self):
        """ See IUberlistingSubtyper
        """
        import pdb; pdb.set_trace()
        if not self.can_enable:
            return self._redirect('Uberlisting View not supported')

        notify(UberlistingWillBeEnabledEvent(self.context))
        alsoProvides(self.context, IUberlistingEnabled)
        notify(UberlistingEnabledEvent(self.context))

        self._redirect(_('Uberlisting View enabled'))

    def disable(self):
        """ See IUberlistingSubtyper
        """
        if not self.can_disable:
            return self._redirect('Uberlisting View not supported')

        notify(UberlistingWillBeDisabledEvent(self.context))
        noLongerProvides(self.context, IUberlistingEnabled)
        notify(UberlistingDisabledEvent(self.context))
        self._redirect(_('Uberlisting View disabled'))
